use tokio::{io::{AsyncReadExt, AsyncWriteExt, BufReader}, net::{tcp::WriteHalf, TcpListener}};
use std::net::SocketAddr;
use crate::{message::{Message, MessageKind}, message_read::MessageReader, server};
use std::convert::Infallible;
use http_body_util::Full;
use hyper::body::Bytes;
use hyper::server::conn::http1;
use hyper::service::service_fn;
use hyper::{Request, Response};
use hyper_util::rt::TokioIo;

pub struct Server {
    pub host: String,
    pub port: u16,
}

impl Server {
    pub fn new(host: impl Into<String>, port: u16) -> Self {
        Server {
            host: host.into(),
            port,
        }
    }

    pub async fn handle_request(_: Request<hyper::body::Incoming>) -> Result<Response<Full<Bytes>>, Infallible> {
        Ok(Response::new(Full::new(Bytes::from("Hello, toto"))))
    }

    #[allow(unreachable_code)]
    pub async fn run(&mut self) -> anyhow::Result<()> {
        let listener = TcpListener::bind(format!(
            "{}:{}",
            self.host,
            self.port
        )).await?;

        println!("Server is running on {}:{}", self.host, self.port);

        loop {
            let (mut stream, addr) = listener.accept().await?;
            println!("Connection received from {}", addr);

            let io = TokioIo::new(stream);

            tokio::task::spawn(async move {
                if let Err(e) = http1::Builder::new()
                    .serve_connection(io, service_fn(Self::handle_request))
                    .await
                {
                    eprintln!("error serving connection: {:?}", e);
                }
            });
        }

        Ok(())
    }
}
